##################################################################################################################
#
# Fertility progression in Gernamy : An application using non-parametric cure survival models
# Add here the reference!
#
# Code writen by V.Bremhorst (Universite catholique de Louvain - Belgium)
# Joint work with : * Michaela Kreyenfeld (Hertie School of Governance Berlin - Max Planck Insitute for 
#									      Demographic Research (Rostock)
#                   * Philippe Lambert (Universite de Liege and Universite catholique de Louvain (Beligum) 
#
# Contact : vincent.bremhorst@uclouvain.be  or vbremhorst@gmail.com 
#
#
#
#     *************************************************************************************************
#     * Plot the baseline survival function with its 95% pointwize credible interval                  *
#     * Plot the population hazard function with its 95% pointwize credible interval                  *
#     *	(when all covariates are set at their reference values)                                       *
#     * Plot the hazard function of the susceptible subjects with its 95% pointwize credible interval *
#     *	(when all covariates are set at their reference values)                                       *
#     * Return the baseline survival and hazard functions                                             *
#     * Return the population hazard and the "susceptible" functions when all covariates are set      *
#     *	at their reference values                                                                     *
#     *************************************************************************************************
#
#
####################################################################################################################


PromotionTime.Baseline.and.refGraphs <- function( newDirectory,
		                                  study.name,		
                		                  Q90, 
		      		                  xl, 
		     		                  xr, 
				                  max_hazard, 
		      		                  beta,
                                                  nbeta,
		     		                  lambda, 
                                                  nlambda,
		      		                  phi, 
		      		                  nknots, 
		      		                  degree, 
				                  timeScale
                                                )

{

   XX <- seq(xl, xr, length = 200)

   BXX <- matrix(.Fortran("cubicBsplines", 
                 x = as.double(XX), 
                 nx = as.integer(200), 
                 xl = as.double(xl), 
                 xr = as.double(xr),
                 ndx = as.integer(nknots), 
                 B = as.double(matrix(0, nrow = 200, ncol = nknots + degree)))$B, ncol = nknots+degree)

   hazardEst <- exp(BXX %*% phi) 

   XXS <- seq(xl, xr, length = 2001)
   uptoL <- c(as.integer(XX[1:199] / (XXS[2]-XXS[1])) + 1, 2000)
   mid <- XXS[1:2000] + (XXS[2]-XXS[1])/2

   BXXS <- matrix(.Fortran("cubicBsplines", 
                  x = as.double(mid), 
                  nx = as.integer(2000), 
                  xl = as.double(xl), 
                  xr = as.double(xr),
                  ndx = as.integer(nknots), 
                  B = as.double(matrix(0, nrow = 2000, ncol = nknots + degree)))$B, ncol = nknots+degree)

   temp <- exp(BXXS %*% (phi))*(XXS[2] - XXS[1])
   survivalEst <- matrix(NA, nrow = 200, ncol = length(phi)/(nknots + degree))

   for(i in 1:200) {

      for(j in 1:(length(phi)/(nknots + degree))) {

         survivalEst[i,j] <- exp(-sum(temp[1:uptoL[i], j]))

      }

   }

   survivalEstMed <- apply(survivalEst, 1, median)
   survivalEst025 <- apply(survivalEst, 1, quantile, 0.025)
   survivalEst975<- apply(survivalEst, 1, quantile, 0.975) 

   filename <- paste("survival", "png", sep=".") 
   png(paste(newDirectory, filename, sep="/"))
   plot(XX, survivalEstMed, xaxt = "n", type = "n", xlab = timeScale, main = paste(study.name, "Baseline survival function", sep = " - "), ylab = " ")

#**************************
# !!! Modify if needed !!!
#**************************
   axis(1, seq(0, max(XX), by = 12), as.character(seq(0, max(XX), by = 12)), cex.axis = 0.75)  

   polygon(c(XX, rev(XX)), c(survivalEst025, rev(survivalEst975)), border = F, col = "grey")
   lines(XX, survivalEstMed, col = 1, lty = 2, lwd = 2)
   dev.off()

   Ref <- hazard.functions( X = matrix(c(1, rep(0, nbeta-1)), nrow = 1),
			    beta = beta, 
			    W = matrix(rep(0, nlambda), nrow = 1), 
   			    lambda = lambda, 
			    iteration = length(phi)/(nknots+degree), 
                            hazardEst = hazardEst, 
                            survivalEst = survivalEst)

   Ref.popMedian <- apply(Ref[[1]], 1 , quantile, 0.5)
   Ref.popLower <- apply(Ref[[1]], 1 , quantile, 0.025)
   Ref.popUpper <- apply(Ref[[1]], 1 , quantile, 0.975)

   Ref.susceptMedian <- apply(Ref[[2]], 1 , quantile, 0.5)
   Ref.susceptLower <- apply(Ref[[2]], 1, quantile, 0.025)
   Ref.susceptUpper <- apply(Ref[[2]], 1, quantile, 0.975)
 
   if(max_hazard == 0) {

      max_hazard <- max( max(apply(Ref[[1]], 1 , quantile, 0.99)[XX <= Q90]), max(apply(Ref[[2]], 1 , quantile, 0.99)[XX <= Q90]))

   }

   filename <- paste("PopHazard", "png", sep=".")
 
   png(paste(newDirectory, filename, sep="/"))
   plot(XX[XX<Q90], Ref.popLower[XX<Q90], xaxt = "n", type = "n", xlab = timeScale, main = paste(study.name, "Baseline population hazard", sep = " - "), ylab = " ", ylim = c(0,max_hazard), xlim = c(0,max(XX[XX<Q90])))

#**************************
# !!! Modify if needed !!!
#**************************
   axis(1, seq(0, max(XX[XX<Q90]), by = 12), as.character(seq(0, max(XX[XX<Q90]), by = 12)), cex.axis = 0.75)  

   polygon(c(XX[XX<Q90], rev(XX[XX<Q90])), c(Ref.popLower[XX<Q90], rev(Ref.popUpper[XX<Q90])), border = F, col = "grey")
   lines(XX[XX<Q90], Ref.popMedian[XX<Q90] , col = 1, lty = 2, lwd = 2)
   dev.off()

   filename <- paste("SusceptHazard", "png", sep=".") 

   png(paste(newDirectory, filename, sep="/"))

   plot(XX[XX<Q90], Ref.susceptLower[XX<Q90], xaxt = "n", type = "n", xlab = timeScale, main = paste(study.name, "Baseline population hazard", sep = " - "), ylab = " ", ylim = c(0,max_hazard), xlim = c(0,max(XX[XX<Q90])))

#**************************
# !!! Modify if needed !!!
#**************************
   axis(1, seq(0, max(XX[XX<Q90]), by = 12), as.character(seq(0, max(XX[XX<Q90]), by = 12)), cex.axis = 0.75)  

   polygon(c(XX[XX<Q90], rev(XX[XX<Q90])), c(Ref.susceptLower[XX<Q90], rev(Ref.susceptUpper[XX<Q90])), border = F, col = "grey")
   lines(XX[XX<Q90], Ref.susceptMedian[XX<Q90] , col = 1, lty = 2, lwd = 2)
   dev.off()

   return(list(hazardEst = hazardEst, 
               survivalEst = survivalEst, 
               Ref.pop  = Ref.popMedian, 
               Ref.suscept = Ref.susceptMedian))


}


