##################################################################################################################
#
# Fertility progression in Gernamy : An application using non-parametric cure survival models
# Add here the reference!
#
# Code writen by V.Bremhorst (Universite catholique de Louvain - Belgium)
# Joint work with : * Michaela Kreyenfeld (Hertie School of Governance Berlin - Max Planck Insitute for 
#									      Demographic Research (Rostock)
#                   * Philippe Lambert (Universite de Liege and Universite catholique de Louvain (Beligum) 
#
# Contact : vincent.bremhorst@uclouvain.be  or vbremhorst@gmail.com 
#
#
#
#	     **********************************************************************
#	     * Perform the Bayesian estimation of the model         		  *
#	     * Return the posterior chains (after burnin) of all model parameters * 
#	     **********************************************************************
#
#
####################################################################################################################


Bayesian.estimation <- function( n, 				
			         obstime,
			         Event,
			         Cure,
			         Cox,
			         nknots,
                                 degree,
                                 order,
		                 iteration,
			         burnin,
                                 phiInitSdprop,
			         betaInitSdprop,
			         lambdaInitSdprop,
			         sd.Cov,
                                 a_delta,
                                 b_delta,
                                 nu
                               ) 

{


###############################################################
# !!!!! Please do not modify the following instructions !!!!! #
###############################################################


# P matrix of the multivariate normal prior distribution.
   penalty <- matrix(.Fortran("penmat", 
                   nknots = as.integer(nknots), 
                   degree = as.integer(degree),
                   order = as.integer(order), 
                   res = as.double(matrix(0, ncol = nknots + degree, nrow = nknots + degree)))$res, ncol = nknots + degree)

   penalty <- penalty + diag(rep(10^(-6), nknots + degree))
   
# Definition of the design matrix X (influencing the probability of the event)

   index_continuous_Cure <- c()
   sd_continuous_Cure <- c()

   X <- rep(1,n)
   colnamesX <- c("Intercept") 

   for(i in 1:length(Cure)) {

      if( class(Cure[[i]]) == "factor" ) {

         X <- cbind(X, model.matrix(~Cure[[i]])[,-1])
         colnamesX <- c( colnamesX, paste(names(Cure)[i], levels(Cure[[i]])[-1], sep = "-") )

      }
    
      else {

         X <- cbind(X, Cure[[i]] / sd( Cure[[i]] ) )
         colnamesX <- c( colnamesX, names(Cure)[i] )
         sd_continuous_Cure <- c( sd_continuous_Cure, sd( Cure[[i]] ) ) 
         index_continuous_Cure <- c( index_continuous_Cure, length(colnamesX) )
         
      }

   }

   cat("\n \n", "Independent variables influencing the probability of the event : ", "\n \n")
   print(colnamesX)
   nbeta <- length(colnamesX) 

# Definition of the design matrix W (influencing the timing of the event for the susceptible subjects)
# Denoted by "Z" in the paper. 

   index_continuous_Cox <- c()
   sd_continuous_Cox <- c() 

   W <- NULL 
   colnamesW <- c() 

   for(i in 1:length(Cox)) {

      if( class(Cox[[i]]) == "factor" ) {

         W <- cbind(W, model.matrix(~Cox[[i]])[,-1])
         colnamesW <- c( colnamesW, paste(names(Cox)[i], levels(Cox[[i]])[-1], sep = "-") )

      }
    
      else {

         W <- cbind(W, Cox[[i]] / sd( Cox[[i]] ) )
         colnamesW <- c( colnamesW, names(Cox)[i] )

         sd_continuous_Cox <- c( sd_continuous_Cox, sd( Cox[[i]] ) ) 
         index_continuous_Cox <- c( index_continuous_Cox, length(colnamesW) )

      }

   }

   cat("\n \n", "Independent variables influencing the timing of the event for the susceptible subjects : ", "\n \n")
   print(colnamesW)
   nlambda <- length(colnamesW)

# Target acceptance rate : 0.23 for multivariate Metropolis step ; 0.44 for univariate Metropolis step

   targetRate <- c(0.23) 

   if (nbeta > 1) { targetRate[2] <- 0.23} 
   else {targetRate[2] <- 0.44}

   if (nlambda > 1) { targetRate[3] <- 0.23} 
   else {targetRate[3] <- 0.44}

# Gestion of the right tail of the distribution. 
# Needed to ensure the convergence of the spline parameters. 
# Do not affect the conclusions of the results.

#*************************************************************************
# !!! This could need a specific traitment in function of the data!!!!
#*************************************************************************

   xl <- 0

   Q995 <- quantile(obstime[Event == 1], 0.995)
   xr <- min(obstime[Event == 1][obstime[Event==1]>=Q995])

   xr <- xr + (min(obstime[Event == 1][obstime[Event ==1] > xr]) -  xr ) /2

   bigger_xr <- which(obstime[Event == 1] > xr)

   obstime[obstime > xr] <- xr
   Event[Event == 1][bigger_xr] <- 0

# Partition of the follow-up interval into small bins 
# Needed for the numerical evaluation of the baseline survival function. 
# (Bremhorst and Lambert 2016)

   bins <- 300
   partition <- seq(xl, xr, length = bins+1)
   width <- partition[2] - partition[1]
   middleBins <- partition[1:bins] + (width/2)

   upto <- as.integer(obstime/width) + 1
   upto[which(upto == bins +1)] <- bins

# Evaluation of the B-splines basis at the middle of each bins

   Bmiddle <- matrix(.Fortran("cubicBsplines", 
                     x = as.double(middleBins), 
                     nx = as.integer(bins), 
                     xl = as.double(xl), 
                     xr = as.double(xr), 
                     ndx = as.integer(nknots), 
                     B = as.double(matrix(0, nrow = bins, ncol = nknots + degree)))$B, ncol = nknots + degree)

# Evaluation of the B-splines basis at the observed failure time.

   Bobs <- matrix(.Fortran("cubicBsplines", 
                  x = as.double(obstime), 
                  nx = as.integer(n), 
                  xl = as.double(xl), 
                  xr = as.double(xr), 
                  ndx = as.integer(nknots), 
                  B = as.double(matrix(0, nrow = n, ncol = nknots + degree)))$B, ncol = nknots + degree)

# Set the mode of the posterior distribution as starting values of the MCMC algorithm
# (for a fixed value of delta)

   param <- rep(0, 1+nknots+degree-1+nbeta+nlambda)

   op <- nlm(log.post.c, param, 
                         Event = Event,
                         nknots = nknots,
                         degree = degree,
                         penalty = penalty,
                         nbeta = nbeta,
                         nlambda = nlambda,
                         Bmiddle = Bmiddle,
                         Bobs = Bobs,   
                         upto = upto,
                         width = width,  
                         X = X, 
                         W = W, 
                         sd.Cov = sd.Cov, 
                         atau = 1, 
                         btau = 0.0001,

             iterlim = 2000, hessian = TRUE)
 
   if( op$code == 1 | op$code == 2) {

      cat("\n \n", "Maximisation of the log-posterior ==> Initial values OK", "\n \n")

      sigma <- solve(op$hessian)
      sigma.phi <- sigma[1:(nknots+degree-1),1:(nknots+degree-1)]

      phi_Init <- c(op$estimate[1:(nknots+degree-1)],10)
      beta_Init <- op$estimate[(nknots+degree-1+1):(nknots+degree-1+nbeta)]
      lambda_Init <- op$estimate[(nknots+degree-1+nbeta+1):(nknots+degree-1+nbeta+nlambda)]
      tau_Init <- exp(op$estimate[length(op$estimate)])
      delta_Init <- a_delta / b_delta # Mean of the gamma prior distribution 

      set.seed(1)

      chain1 = .Fortran("PromTimeRightCensoring", 

                degree = as.integer(degree), 
                nknots = as.integer(nknots),

                n = as.integer(n),
                obstime = as.double(obstime), 
                Event = as.integer(Event),
 
                X = as.double(X),
                W = as.double(W), 
	        nbeta = as.integer(nbeta),
                nlambda = as.integer(nlambda),          

                xl = as.double(xl), 
                xr = as.double(xr),
                Npart = as.integer(bins),
                middle = as.double(middleBins), 
                upto = as.integer(upto), 
                width = as.double(width), 

                iteration = as.integer(iteration),
                burnin = as.integer(burnin), 

                nu = as.double(nu),
                initdelta = as.double(delta_Init), 
                delta = as.double(rep(0, iteration+1)),
                a_delta = as.double(a_delta), 
                b_delta = as.double(b_delta), 

                phiInitSDprop = as.double(phiInitSdprop),
                betaInitSDprop = as.double(betaInitSdprop), 
                lambdaInitSDprop = as.double(lambdaInitSdprop),
 
                initphi = as.double(phi_Init),
                initbeta = as.double(beta_Init),
                initlambda = as.double(lambda_Init),

                sd_Cov = as.double(sd.Cov),

                targetRate = as.double(targetRate), 
                accept = as.integer(matrix(0, ncol = iteration, nrow = 3)),
 
                phi = as.double(matrix(0, ncol =  iteration, nrow = nknots + degree)), 
                beta = as.double(matrix(0, ncol = iteration, nrow = nbeta)), 
                lambda = as.double(matrix(0, ncol = iteration, nrow = nlambda)), 
 
                sigmaphi = as.double(sigma.phi),
                inittau = as.double(tau_Init), 
                tau = as.double(rep(0, iteration+1)), 
                penalty = as.double(penalty))

      cat("First posterior chain completed \n \n") 

      set.seed(2)

      chain2 = .Fortran("PromTimeRightCensoring", 

                degree = as.integer(degree), 
                nknots = as.integer(nknots),

                n = as.integer(n),
                obstime = as.double(obstime), 
                Event = as.integer(Event), 
 
                X = as.double(X),
                W = as.double(W), 
                nbeta = as.integer(nbeta),
                nlambda = as.integer(nlambda),          
                xl = as.double(xl), 
                xr = as.double(xr),
                Npart = as.integer(bins),
                middle = as.double(middleBins), 
                upto = as.integer(upto), 
                width = as.double(width), 

                iteration = as.integer(iteration),
                burnin = as.integer(burnin), 

                nu = as.double(nu),
                initdelta = as.double(delta_Init), 
                delta = as.double(rep(0, iteration+1)),
                a_delta = as.double(a_delta), 
                b_delta = as.double(b_delta), 

                phiInitSDprop = as.double(phiInitSdprop),
                betaInitSDprop = as.double(betaInitSdprop), 
                lambdaInitSDprop = as.double(lambdaInitSdprop),

                initphi = as.double(phi_Init),
                initbeta = as.double(beta_Init),
                initlambda = as.double(lambda_Init),

                sd_Cov = as.double(sd.Cov),

                targetRate = as.double(targetRate), 
                accept = as.integer(matrix(0, ncol = iteration, nrow = 3)),
 
                phi = as.double(matrix(0, ncol =  iteration, nrow = nknots + degree)), 
                beta = as.double(matrix(0, ncol = iteration, nrow = nbeta)), 
                lambda = as.double(matrix(0, ncol = iteration, nrow = nlambda)), 
 
                sigmaphi = as.double(sigma.phi),
                inittau = as.double(tau_Init), 
                tau = as.double(rep(0, iteration+1)), 
                penalty = as.double(penalty))

      cat("Second posterior chain completed \n \n") 


      accept <- NULL 
      accept <- rbind(accept, apply( matrix(chain1$accept, ncol = iteration)[,-c(1:burnin)], 1, mean))
      accept <- rbind(accept, apply( matrix(chain2$accept, ncol = iteration)[,-c(1:burnin)], 1, mean)) 
      dimnames(accept) <- list( list("chain 1", "chain 2"), list("phi", "beta", "lambda")) 
      
      phiPost <- matrix(chain1$phi, ncol = iteration)[,-c(1:burnin)]
      betaPost <- matrix(chain1$beta, ncol = iteration)[,-c(1:burnin)]
      lambdaPost <- matrix(chain1$lambda, ncol = iteration)[,-c(1:burnin)]
      tauPost <- chain1$tau[-c(1:(burnin+1))]
      deltaPost <- chain1$delta[-c(1:(burnin+1))]

      phiPost10_1 <- matrix(0, ncol = dim(phiPost)[2]/10, nrow = nknots+degree)
      betaPost10_1 <- matrix(0, ncol = dim(betaPost)[2]/10, nrow = nbeta)
      lambdaPost10_1 <- matrix(0, ncol = dim(lambdaPost)[2]/10, nrow = nlambda)
      tauPost10_1 <- c()
      deltaPost10_1 <- c()

      for(i in 1:(dim(phiPost)[2]/10)) {
       
         phiPost10_1[,i] = phiPost[,((i-1)*10) +1]
         betaPost10_1[,i] = betaPost[,((i-1)*10) +1]
         lambdaPost10_1[,i] = lambdaPost[,((i-1)*10) +1]  
         tauPost10_1[i] = tauPost[((i-1)*10) +1]
         deltaPost10_1[i] = deltaPost[((i-1)*10) +1]

      }
    
      phiPost <- matrix(chain2$phi, ncol = iteration)[,-c(1:burnin)]
      betaPost <- matrix(chain2$beta, ncol = iteration)[,-c(1:burnin)]
      lambdaPost <- matrix(chain2$lambda, ncol = iteration)[,-c(1:burnin)]
      tauPost <- chain2$tau[-c(1:(burnin+1))]
      deltaPost <- chain2$delta[-c(1:(burnin+1))]

      phiPost10_2 <- matrix(0, ncol = dim(phiPost)[2]/10, nrow = nknots+degree)
      betaPost10_2 <- matrix(0, ncol = dim(betaPost)[2]/10, nrow = nbeta)
      lambdaPost10_2 <- matrix(0, ncol = dim(lambdaPost)[2]/10, nrow = nlambda)
      tauPost10_2 <- c()
      deltaPost10_2 <- c()

      for(i in 1:(dim(phiPost)[2]/10)) {
       
         phiPost10_2[,i] = phiPost[,((i-1)*10) +1]
         betaPost10_2[,i] = betaPost[,((i-1)*10) +1]
         lambdaPost10_2[,i] = lambdaPost[,((i-1)*10) +1]  
         tauPost10_2[i] = tauPost[((i-1)*10) +1]
         deltaPost10_2[i] = deltaPost[((i-1)*10) +1]

      }

      for(i in 1:length(sd_continuous_Cure)) {

         betaPost10_1[index_continuous_Cure[i],] <- betaPost10_1[index_continuous_Cure[i],] / sd_continuous_Cure[i] 

         betaPost10_2[index_continuous_Cure[i],] <- betaPost10_2[index_continuous_Cure[i],] / sd_continuous_Cure[i] 

      }


      for(i in 1:length(sd_continuous_Cox)) {

         lambdaPost10_1[index_continuous_Cox[i],] <- lambdaPost10_1[index_continuous_Cox[i],] / sd_continuous_Cox[i] 

         lambdaPost10_2[index_continuous_Cox[i],] <- lambdaPost10_2[index_continuous_Cox[i],] / sd_continuous_Cox[i] 

      }

      return( list( xl = xl, 
                    xr = xr, 
                    nbeta = nbeta, 
                    beta = list(chain1 = betaPost10_1, chain2 = betaPost10_2),
                    colnamesX = colnamesX,
                    nlambda = nlambda,
                    lambda = list(chain1 = lambdaPost10_1, chain2 = lambdaPost10_2),
                    colnamesW = colnamesW, 
                    phi = list(chain1 = phiPost10_1, chain2 = phiPost10_2), 
                    tau = list(chain1 = tauPost10_1, chain2 = tauPost10_2),
                    delta = list(chain1 = deltaPost10_1, chain2 = deltaPost10_2), 
		    accept = accept )
            )

   }

}

