##################################################################################################################
#
# Fertility progression in Gernamy : An application using non-parametric cure survival models
# Add here the reference!
#
# Code writen by V.Bremhorst (Universite catholique de Louvain - Belgium)
# Joint work with : * Michaela Kreyenfeld (Hertie School of Governance Berlin - Max Planck Insitute for 
#						 			      Demographic Research (Rostock)
#                   * Philippe Lambert (Universite de Liege and Universite catholique de Louvain (Beligum) 
#
# Contact : vincent.bremhorst@uclouvain.be  or vbremhorst@gmail.com 
#
#
#
#	*******************************************************************************************
# 	* Main function.                                                                          *
# 	* Produce the numerical results, the graphs presented in the paper.                       *
#	*******************************************************************************************
#
####################################################################################################################

PromotionTime.Bremhorst.et.al2016 <- function( study.name,
      					       n,
                                               obstime,
                                               Event,
					       Cure,
                                               Cox,
                                               Graphs = TRUE,
                                               hazard.profile = list(),
                                               variables = NULL,
                                               max_hazard = 0,
                                               post.probabilities = list(),
                                               timeScale = "time",
                                               nknots = 12,
                                               degree = 3,
                                               order = 3,
		                               iteration = 150000,
                                               burnin = 50000,
                                               phiInitSdprop = 0.1,
                                               betaInitSdprop = 0.1,
				 	       lambdaInitSdprop = 0.1,	
 					       sd.Cov = 10,
				               a_delta = 0.0001,
					       b_delta = 0.0001,
 					       nu = 2
					     ) 


{ 

   newDirectory <- paste(getwd(), paste(study.name, "Results", sep = "-"), sep = "/") 
   dir.create(newDirectory)

# Kaplan Meier estimate 

   surv_data <- Surv(obstime, Event)
   filename <- paste(study.name, paste("KaplanMeier", "png", sep ="."), sep = "-")
   png(paste(newDirectory, filename, sep = "/")) 
   KM <- survfit(surv_data~1, conf.int = NULL)
   plot(KM, mark.time = FALSE, yaxt = "n", xaxt = "n", main = study.name, ylab = "S(x)", xlab = timeScale)

#**************************
# !!! Modify if needed !!!
#**************************

   abline(h = 0.25, lwd = 1, lty = 2, col = "lightgrey")
   abline(h = 0.5, lwd = 1, lty = 2, col = "lightgrey")
   abline(h = 0.75, lwd = 1, lty = 2, col = "lightgrey")
   axis(2, c(0,0.25,0.5,0.75,1), c("0", "0.25", "0.5", "0.75", "1"))
   axis(1, seq(0, max(obstime), by = 12), as.character(seq(0, max(obstime), by = 12)), cex.axis = 0.75)  
   dev.off()

   posterior.chains <- Bayesian.estimation( n = n, 
		                            obstime = obstime, 
                        	            Event = Event, 
	                                    Cure = Cure, 
	                                    Cox = Cox, 
                                            nknots = nknots,
                                            degree = degree, 
                                            order = order, 
                                            iteration = iteration, 
                                            burnin = burnin, 
                                            phiInitSdprop = phiInitSdprop, 
                                            betaInitSdprop = betaInitSdprop, 
                                            lambdaInitSdprop = lambdaInitSdprop, 
                                            sd.Cov = sd.Cov, 
                                            a_delta = a_delta, 
                                            b_delta = b_delta, 
                                            nu = nu)


   newDirectory <- paste(getwd(), paste(study.name, "Convergence issues", sep = "-"), sep = "/") 
   dir.create(newDirectory)

   Convergence.issues( nbeta = posterior.chains[["nbeta"]],
                      beta = posterior.chains[["beta"]], 
                      nlambda = posterior.chains[["nlambda"]], 
                      lambda = posterior.chains[["lambda"]],
                      nspline = nknots+degree-1, 
                      phi = posterior.chains[["phi"]], 
                      tau = posterior.chains[["tau"]],
		      delta = posterior.chains[["delta"]], 
                      accept = posterior.chains[["accept"]], 
                      newDirectory = newDirectory 
                     )

   phiPost <- cbind(posterior.chains[["phi"]][[1]], posterior.chains[["phi"]][[2]])
   betaPost <-  cbind(posterior.chains[["beta"]][[1]], posterior.chains[["beta"]][[2]])
   lambdaPost <-  cbind(posterior.chains[["lambda"]][[1]], posterior.chains[["lambda"]][[2]])

   newDirectory <- paste(getwd(), paste(study.name, "Results", sep = "-"), sep = "/") 

   Nresults <- Numerical.results( nbeta = posterior.chains[["nbeta"]], 
                                  beta = betaPost, 
                                  colnamesX = posterior.chains[["colnamesX"]], 
                                  nlambda = posterior.chains[["nlambda"]], 
                                  lambda = lambdaPost, 
                                  colnamesW = posterior.chains[["colnamesW"]], 
                                  newDirectory = newDirectory
                                )

   if(Graphs) {

      Baseline.and.Ref <- PromotionTime.Baseline.and.refGraphs( 
						newDirectory = newDirectory,
			                        study.name = study.name,
				                Q90 = quantile(obstime[Event == 1], 0.9),  
				                xl = posterior.chains[["xl"]], 
				                xr = posterior.chains[["xr"]], 
				                max_hazard = max_hazard, 
				                beta = betaPost,
  				                nbeta = posterior.chains[["nbeta"]],
				                lambda = lambdaPost, 
				                nlambda = posterior.chains[["nlambda"]],
				                phi = phiPost,
				                nknots = nknots, 
				                degree = degree, 
				                timeScale = timeScale
                                                )

      if(length(hazard.profile) > 0) {

         PromotionTime.Graphs( newDirectory = newDirectory,
			       study.name = study.name,
			       Q90 = quantile(obstime[Event == 1], 0.9),  
			       xl = posterior.chains[["xl"]], 
			       xr = posterior.chains[["xr"]], 
    		               max_hazard = max_hazard, 
                               hazardEst = Baseline.and.Ref[["hazardEst"]], 
			       survivalEst = Baseline.and.Ref[["survivalEst"]], 
 			       Ref.pop = Baseline.and.Ref[["Ref.pop"]],
			       Ref.suscept = Baseline.and.Ref[["Ref.suscept"]], 
                               hazard.profile = hazard.profile,
                               variables = variables,
                               colnamesX = posterior.chains[["colnamesX"]],
			       colnamesW = posterior.chains[["colnamesW"]], 
			       beta = betaPost, 
			       nbeta = posterior.chains[["nbeta"]], 
			       lambda = lambdaPost, 
			       nlambda = posterior.chains[["nlambda"]], 
			       timeScale = timeScale
                              )
      }

   }

   if(length(post.probabilities) > 0) {      

      posterior.probabilities( post.probabilities, 
                               nbeta = posterior.chains[["nbeta"]], 
                               beta = betaPost, 
                               colnamesX = posterior.chains[["colnamesX"]], 
                               newDirectory
                             )

   }

}


